@@ -1,24 +1,21 @@
 import datetime
-from builtins import object
 from logging import getLogger
 from typing import Any, AnyStr, Dict, List, Optional, Tuple, Union, Iterator
 
+import click
 import gridfs
 import pymongo
 from gridfs import NoFile
 
-from notebooker.constants import (
-    JobStatus,
-    NotebookResultBase,
-    NotebookResultComplete,
-    NotebookResultError,
-    NotebookResultPending,
-)
+from notebooker.constants import JobStatus, NotebookResultComplete, NotebookResultError, NotebookResultPending
 
 logger = getLogger(__name__)
+REMOVE_ID_PROJECTION = {"_id": 0}
+REMOVE_PAYLOAD_FIELDS_PROJECTION = {"raw_html_resources": 0, "raw_html": 0, "raw_ipynb_json": 0}
+REMOVE_PAYLOAD_FIELDS_AND_ID_PROJECTION = dict(REMOVE_PAYLOAD_FIELDS_PROJECTION, **REMOVE_ID_PROJECTION)
 
 
-class NotebookResultSerializer(object):
+class MongoResultSerializer:
     # This class is the interface between Mongo and the rest of the application
 
     def __init__(self, database_name="notebooker", mongo_host="localhost", result_collection_name="NOTEBOOK_OUTPUT"):
@@ -29,6 +26,32 @@ class NotebookResultSerializer(object):
         self.library = mongo_connection[result_collection_name]
         self.result_data_store = gridfs.GridFS(mongo_connection, "notebook_data")
 
+    def __init_subclass__(cls, cli_options: click.Command = None, **kwargs):
+        if cli_options is None:
+            raise ValueError(
+                "A MongoResultSerializer has been declared without cli_options. "
+                "Please add them like so: `class MySerializer(cli_options=cli_opts)`."
+            )
+        cls.cli_options = cli_options
+        super().__init_subclass__(**kwargs)
+
+    def serializer_args_to_cmdline_args(self) -> List[str]:
+        args = []
+        for cli_arg in self.cli_options.params:
+            if not hasattr(self, cli_arg.name):
+                raise ValueError(
+                    "The Serializer class must have attributes which are named the same as the click "
+                    "options, e.g. --mongo-database should have a 'mongo_database' attribute"
+                )
+            opt, value = cli_arg.opts[0], getattr(self, cli_arg.name)
+            if value is not None:
+                args.extend([opt, value])
+        return args
+
+    @classmethod
+    def get_name(cls):
+        return cls.__name__
+
     def get_mongo_database(self):
         raise NotImplementedError()
 
@@ -42,7 +65,7 @@ class NotebookResultSerializer(object):
         # Ensure that the job_id index exists
         self.library.create_index([("job_id", pymongo.ASCENDING)], background=True)
         self.library.create_index([("report_name", pymongo.ASCENDING)], background=True)
-        self.library.create_index([('update_time', pymongo.DESCENDING)], background=True)
+        self.library.create_index([("update_time", pymongo.DESCENDING)], background=True)
         self.library.create_index([("status", pymongo.ASCENDING), ("update_time", pymongo.DESCENDING)], background=True)
 
     def _save_to_db(self, notebook_result):
@@ -77,6 +100,7 @@ class NotebookResultSerializer(object):
         overrides: Optional[Dict] = None,
         mailto: str = "",
         generate_pdf_output: bool = True,
+        hide_code: bool = False,
     ) -> None:
         """ Call this when we are just starting a check. Saves a "pending" job into storage. """
         job_start_time = job_start_time or datetime.datetime.now()
@@ -90,6 +114,7 @@ class NotebookResultSerializer(object):
             mailto=mailto,
             generate_pdf_output=generate_pdf_output,
             overrides=overrides or {},
+            hide_code=hide_code,
         )
         self._save_to_db(pending_result)
 
@@ -156,11 +181,13 @@ class NotebookResultSerializer(object):
                 raw_html_resources=result.get("raw_html_resources", {}),
                 raw_ipynb_json=result.get("raw_ipynb_json"),
                 raw_html=result.get("raw_html"),
+                email_html=result.get("email_html"),
                 pdf=result.get("pdf", ""),
                 overrides=result.get("overrides", {}),
                 generate_pdf_output=result.get("generate_pdf_output", True),
                 report_title=result.get("report_title", result["report_name"]),
                 mailto=result.get("mailto", ""),
+                hide_code=result.get("hide_code", False),
                 stdout=result.get("stdout", []),
             )
         elif cls == NotebookResultPending:
@@ -174,6 +201,7 @@ class NotebookResultSerializer(object):
                 generate_pdf_output=result.get("generate_pdf_output", True),
                 report_title=result.get("report_title", result["report_name"]),
                 mailto=result.get("mailto", ""),
+                hide_code=result.get("hide_code", False),
                 stdout=result.get("stdout", []),
             )
 
@@ -189,6 +217,7 @@ class NotebookResultSerializer(object):
                 generate_pdf_output=result.get("generate_pdf_output", True),
                 report_title=result.get("report_title", result["report_name"]),
                 mailto=result.get("mailto", ""),
+                hide_code=result.get("hide_code", False),
                 stdout=result.get("stdout", []),
             )
         else:
@@ -212,9 +241,7 @@ class NotebookResultSerializer(object):
             base_filter.update(mongo_filter)
         if since:
             base_filter.update({"update_time": {"$gt": since}})
-        projection = (
-            {"_id": 0} if load_payload else {"raw_html_resources": 0, "raw_html": 0, "raw_ipynb_json": 0, "_id": 0}
-        )
+        projection = REMOVE_ID_PROJECTION if load_payload else REMOVE_PAYLOAD_FIELDS_AND_ID_PROJECTION
         results = self.library.find(base_filter, projection).sort("update_time", -1).limit(limit)
         for res in results:
             if res:
@@ -227,8 +254,19 @@ class NotebookResultSerializer(object):
         base_filter = {"status": {"$ne": JobStatus.DELETED.value}}
         if mongo_filter:
             base_filter.update(mongo_filter)
-        projection = {"report_name": 1, "job_id": 1, "_id": 0}
-        for result in self.library.find(base_filter, projection).sort("update_time", -1).limit(limit):
+        results = self.library.aggregate(
+            [
+                stage
+                for stage in (
+                    {"$match": base_filter},
+                    {"$sort": {"update_time": -1}},
+                    {"$limit": limit} if limit else {},
+                    {"$project": {"report_name": 1, "job_id": 1}},
+                )
+                if stage
+            ]
+        )
+        for result in results:
             keys.append((result["report_name"], result["job_id"]))
         return keys
 
@@ -287,6 +325,7 @@ class NotebookResultSerializer(object):
         results = self.library.aggregate(
             [
                 {"$match": mongo_filter},
+                {"$project": REMOVE_PAYLOAD_FIELDS_PROJECTION},
                 {"$sort": {"update_time": -1}},
                 {"$group": {"_id": "$overrides", "job_id": {"$first": "$job_id"}}},
             ]
